# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..maxcurvature import maxcurvature


def test_maxcurvature_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        image=dict(
            argstr="--image %s",
            extensions=None,
        ),
        output=dict(
            argstr="--output %s",
            hash_files=False,
        ),
        sigma=dict(
            argstr="--sigma %f",
        ),
        verbose=dict(
            argstr="--verbose ",
        ),
    )
    inputs = maxcurvature.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_maxcurvature_outputs():
    output_map = dict(
        output=dict(
            extensions=None,
        ),
    )
    outputs = maxcurvature.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
