# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Nitime is a library for time-series analysis of data from neuroscience experiments."""
from .analysis import (
    CoherenceAnalyzerInputSpec,
    CoherenceAnalyzerOutputSpec,
    CoherenceAnalyzer,
)
