# Contrast & F-tests mode
# real : control real EVs
# orig : control original EVs
set fmri(con_mode_old) orig
set fmri(con_mode) orig

### Needs iteration

# Display images for contrast_real 1
set fmri(conpic_real.1) 1

# Title for contrast_real 1
set fmri(conname_real.1) "left>right"

# Real contrast_real vector 1 element 1
set fmri(con_real1.1) 1

# Real contrast_real vector 1 element 2
set fmri(con_real1.2) -1.0

# Real contrast_real vector 1 element 3
set fmri(con_real1.3) 1.0

# Real contrast_real vector 1 element 4
set fmri(con_real1.4) -1.0

# Real contrast_real vector 1 element 5
set fmri(con_real1.5) 1.0

# Real contrast_real vector 1 element 6
set fmri(con_real1.6) -1.0

# Real contrast_real vector 1 element 7
set fmri(con_real1.7) 1.0

# Real contrast_real vector 1 element 8
set fmri(con_real1.8) -1.0

# Display images for contrast_real 2
set fmri(conpic_real.2) 1

# Title for contrast_real 2
set fmri(conname_real.2) "visual>vibe"

# Real contrast_real vector 2 element 1
set fmri(con_real2.1) -1.0

# Real contrast_real vector 2 element 2
set fmri(con_real2.2) -1.0

# Real contrast_real vector 2 element 3
set fmri(con_real2.3) -1.0

# Real contrast_real vector 2 element 4
set fmri(con_real2.4) -1.0

# Real contrast_real vector 2 element 5
set fmri(con_real2.5) 1.0

# Real contrast_real vector 2 element 6
set fmri(con_real2.6) 1.0

# Real contrast_real vector 2 element 7
set fmri(con_real2.7) 1.0

# Real contrast_real vector 2 element 8
set fmri(con_real2.8) 1.0

# Display images for contrast_real 3
set fmri(conpic_real.3) 1

# Title for contrast_real 3
set fmri(conname_real.3) "seq>all"

# Real contrast_real vector 3 element 1
set fmri(con_real3.1) -1.0

# Real contrast_real vector 3 element 2
set fmri(con_real3.2) -1.0

# Real contrast_real vector 3 element 3
set fmri(con_real3.3) 1.0

# Real contrast_real vector 3 element 4
set fmri(con_real3.4) 1.0

# Real contrast_real vector 3 element 5
set fmri(con_real3.5) -1.0

# Real contrast_real vector 3 element 6
set fmri(con_real3.6) -1.0

# Real contrast_real vector 3 element 7
set fmri(con_real3.7) 1.0

# Real contrast_real vector 3 element 8
set fmri(con_real3.8) 1.0

# Display images for contrast_real 4
set fmri(conpic_real.4) 1

# Title for contrast_real 4
set fmri(conname_real.4) "visual seq>all"

# Real contrast_real vector 4 element 1
set fmri(con_real4.1) 0

# Real contrast_real vector 4 element 2
set fmri(con_real4.2) 0

# Real contrast_real vector 4 element 3
set fmri(con_real4.3) 0

# Real contrast_real vector 4 element 4
set fmri(con_real4.4) 0

# Real contrast_real vector 4 element 5
set fmri(con_real4.5) -1.0

# Real contrast_real vector 4 element 6
set fmri(con_real4.6) -1.0

# Real contrast_real vector 4 element 7
set fmri(con_real4.7) 1.0

# Real contrast_real vector 4 element 8
set fmri(con_real4.8) 1.0

# Display images for contrast_real 5
set fmri(conpic_real.5) 1

# Title for contrast_real 5
set fmri(conname_real.5) "vibe seq>all"

# Real contrast_real vector 5 element 1
set fmri(con_real5.1) -1.0

# Real contrast_real vector 5 element 2
set fmri(con_real5.2) -1.0

# Real contrast_real vector 5 element 3
set fmri(con_real5.3) 1.0

# Real contrast_real vector 5 element 4
set fmri(con_real5.4) 1.0

# Real contrast_real vector 5 element 5
set fmri(con_real5.5) 0

# Real contrast_real vector 5 element 6
set fmri(con_real5.6) 0

# Real contrast_real vector 5 element 7
set fmri(con_real5.7) 0

# Real contrast_real vector 5 element 8
set fmri(con_real5.8) 0

# Display images for contrast_real 6
set fmri(conpic_real.6) 1

# Title for contrast_real 6
set fmri(conname_real.6) "visual seq>vibe seq"

# Real contrast_real vector 6 element 1
set fmri(con_real6.1) 0

# Real contrast_real vector 6 element 2
set fmri(con_real6.2) 0

# Real contrast_real vector 6 element 3
set fmri(con_real6.3) -1.0

# Real contrast_real vector 6 element 4
set fmri(con_real6.4) -1.0

# Real contrast_real vector 6 element 5
set fmri(con_real6.5) 0

# Real contrast_real vector 6 element 6
set fmri(con_real6.6) 0

# Real contrast_real vector 6 element 7
set fmri(con_real6.7) 1.0

# Real contrast_real vector 6 element 8
set fmri(con_real6.8) 1.0

# Display images for contrast_real 7
set fmri(conpic_real.7) 1

# Title for contrast_real 7
set fmri(conname_real.7) "visual all>vibe all"

# Real contrast_real vector 7 element 1
set fmri(con_real7.1) -1.0

# Real contrast_real vector 7 element 2
set fmri(con_real7.2) -1.0

# Real contrast_real vector 7 element 3
set fmri(con_real7.3) 0

# Real contrast_real vector 7 element 4
set fmri(con_real7.4) 0

# Real contrast_real vector 7 element 5
set fmri(con_real7.5) 1.0

# Real contrast_real vector 7 element 6
set fmri(con_real7.6) 1.0

# Real contrast_real vector 7 element 7
set fmri(con_real7.7) 0

# Real contrast_real vector 7 element 8
set fmri(con_real7.8) 0

# Display images for contrast_real 8
set fmri(conpic_real.8) 1

# Title for contrast_real 8
set fmri(conname_real.8) "mode x complexity"

# Real contrast_real vector 8 element 1
set fmri(con_real8.1) -1.0

# Real contrast_real vector 8 element 2
set fmri(con_real8.2) -1.0

# Real contrast_real vector 8 element 3
set fmri(con_real8.3) 1.0

# Real contrast_real vector 8 element 4
set fmri(con_real8.4) 1.0

# Real contrast_real vector 8 element 5
set fmri(con_real8.5) 1.0

# Real contrast_real vector 8 element 6
set fmri(con_real8.6) 1.0

# Real contrast_real vector 8 element 7
set fmri(con_real8.7) -1.0

# Real contrast_real vector 8 element 8
set fmri(con_real8.8) -1.0

# Display images for contrast_orig 1
set fmri(conpic_orig.1) 1

# Title for contrast_orig 1
set fmri(conname_orig.1) "left>right"

# Real contrast_orig vector 1 element 1
set fmri(con_orig1.1) 1

# Real contrast_orig vector 1 element 2
set fmri(con_orig1.2) -1.0

# Real contrast_orig vector 1 element 3
set fmri(con_orig1.3) 1.0

# Real contrast_orig vector 1 element 4
set fmri(con_orig1.4) -1.0

# Real contrast_orig vector 1 element 5
set fmri(con_orig1.5) 1.0

# Real contrast_orig vector 1 element 6
set fmri(con_orig1.6) -1.0

# Real contrast_orig vector 1 element 7
set fmri(con_orig1.7) 1.0

# Real contrast_orig vector 1 element 8
set fmri(con_orig1.8) -1.0

# Display images for contrast_orig 2
set fmri(conpic_orig.2) 1

# Title for contrast_orig 2
set fmri(conname_orig.2) "visual>vibe"

# Real contrast_orig vector 2 element 1
set fmri(con_orig2.1) -1.0

# Real contrast_orig vector 2 element 2
set fmri(con_orig2.2) -1.0

# Real contrast_orig vector 2 element 3
set fmri(con_orig2.3) -1.0

# Real contrast_orig vector 2 element 4
set fmri(con_orig2.4) -1.0

# Real contrast_orig vector 2 element 5
set fmri(con_orig2.5) 1.0

# Real contrast_orig vector 2 element 6
set fmri(con_orig2.6) 1.0

# Real contrast_orig vector 2 element 7
set fmri(con_orig2.7) 1.0

# Real contrast_orig vector 2 element 8
set fmri(con_orig2.8) 1.0

# Display images for contrast_orig 3
set fmri(conpic_orig.3) 1

# Title for contrast_orig 3
set fmri(conname_orig.3) "seq>all"

# Real contrast_orig vector 3 element 1
set fmri(con_orig3.1) -1.0

# Real contrast_orig vector 3 element 2
set fmri(con_orig3.2) -1.0

# Real contrast_orig vector 3 element 3
set fmri(con_orig3.3) 1.0

# Real contrast_orig vector 3 element 4
set fmri(con_orig3.4) 1.0

# Real contrast_orig vector 3 element 5
set fmri(con_orig3.5) -1.0

# Real contrast_orig vector 3 element 6
set fmri(con_orig3.6) -1.0

# Real contrast_orig vector 3 element 7
set fmri(con_orig3.7) 1.0

# Real contrast_orig vector 3 element 8
set fmri(con_orig3.8) 1.0

# Display images for contrast_orig 4
set fmri(conpic_orig.4) 1

# Title for contrast_orig 4
set fmri(conname_orig.4) "visual seq>all"

# Real contrast_orig vector 4 element 1
set fmri(con_orig4.1) 0

# Real contrast_orig vector 4 element 2
set fmri(con_orig4.2) 0

# Real contrast_orig vector 4 element 3
set fmri(con_orig4.3) 0

# Real contrast_orig vector 4 element 4
set fmri(con_orig4.4) 0

# Real contrast_orig vector 4 element 5
set fmri(con_orig4.5) -1.0

# Real contrast_orig vector 4 element 6
set fmri(con_orig4.6) -1.0

# Real contrast_orig vector 4 element 7
set fmri(con_orig4.7) 1.0

# Real contrast_orig vector 4 element 8
set fmri(con_orig4.8) 1.0

# Display images for contrast_orig 5
set fmri(conpic_orig.5) 1

# Title for contrast_orig 5
set fmri(conname_orig.5) "vibe seq>all"

# Real contrast_orig vector 5 element 1
set fmri(con_orig5.1) -1.0

# Real contrast_orig vector 5 element 2
set fmri(con_orig5.2) -1.0

# Real contrast_orig vector 5 element 3
set fmri(con_orig5.3) 1.0

# Real contrast_orig vector 5 element 4
set fmri(con_orig5.4) 1.0

# Real contrast_orig vector 5 element 5
set fmri(con_orig5.5) 0

# Real contrast_orig vector 5 element 6
set fmri(con_orig5.6) 0

# Real contrast_orig vector 5 element 7
set fmri(con_orig5.7) 0

# Real contrast_orig vector 5 element 8
set fmri(con_orig5.8) 0

# Display images for contrast_orig 6
set fmri(conpic_orig.6) 1

# Title for contrast_orig 6
set fmri(conname_orig.6) "visual seq>vibe seq"

# Real contrast_orig vector 6 element 1
set fmri(con_orig6.1) 0

# Real contrast_orig vector 6 element 2
set fmri(con_orig6.2) 0

# Real contrast_orig vector 6 element 3
set fmri(con_orig6.3) -1.0

# Real contrast_orig vector 6 element 4
set fmri(con_orig6.4) -1.0

# Real contrast_orig vector 6 element 5
set fmri(con_orig6.5) 0

# Real contrast_orig vector 6 element 6
set fmri(con_orig6.6) 0

# Real contrast_orig vector 6 element 7
set fmri(con_orig6.7) 1.0

# Real contrast_orig vector 6 element 8
set fmri(con_orig6.8) 1.0

# Display images for contrast_orig 7
set fmri(conpic_orig.7) 1

# Title for contrast_orig 7
set fmri(conname_orig.7) "visual all>vibe all"

# Real contrast_orig vector 7 element 1
set fmri(con_orig7.1) -1.0

# Real contrast_orig vector 7 element 2
set fmri(con_orig7.2) -1.0

# Real contrast_orig vector 7 element 3
set fmri(con_orig7.3) 0

# Real contrast_orig vector 7 element 4
set fmri(con_orig7.4) 0

# Real contrast_orig vector 7 element 5
set fmri(con_orig7.5) 1.0

# Real contrast_orig vector 7 element 6
set fmri(con_orig7.6) 1.0

# Real contrast_orig vector 7 element 7
set fmri(con_orig7.7) 0

# Real contrast_orig vector 7 element 8
set fmri(con_orig7.8) 0

# Display images for contrast_orig 8
set fmri(conpic_orig.8) 1

# Title for contrast_orig 8
set fmri(conname_orig.8) "mode x complexity"

# Real contrast_orig vector 8 element 1
set fmri(con_orig8.1) -1.0

# Real contrast_orig vector 8 element 2
set fmri(con_orig8.2) -1.0

# Real contrast_orig vector 8 element 3
set fmri(con_orig8.3) 1.0

# Real contrast_orig vector 8 element 4
set fmri(con_orig8.4) 1.0

# Real contrast_orig vector 8 element 5
set fmri(con_orig8.5) 1.0

# Real contrast_orig vector 8 element 6
set fmri(con_orig8.6) 1.0

# Real contrast_orig vector 8 element 7
set fmri(con_orig8.7) -1.0

# Real contrast_orig vector 8 element 8
set fmri(con_orig8.8) -1.0

### This is fixed

# Contrast masking - use >0 instead of thresholding?
set fmri(conmask_zerothresh_yn) 0

### These are set for the full combo of contrasts - needs iteration

# Mask real contrast/F-test 1 with real contrast/F-test 2?
set fmri(conmask1_2) 0

# Mask real contrast/F-test 1 with real contrast/F-test 3?
set fmri(conmask1_3) 0

# Mask real contrast/F-test 1 with real contrast/F-test 4?
set fmri(conmask1_4) 0

# Mask real contrast/F-test 1 with real contrast/F-test 5?
set fmri(conmask1_5) 0

# Mask real contrast/F-test 1 with real contrast/F-test 6?
set fmri(conmask1_6) 0

# Mask real contrast/F-test 1 with real contrast/F-test 7?
set fmri(conmask1_7) 0

# Mask real contrast/F-test 1 with real contrast/F-test 8?
set fmri(conmask1_8) 0

# Mask real contrast/F-test 2 with real contrast/F-test 1?
set fmri(conmask2_1) 0

# Mask real contrast/F-test 2 with real contrast/F-test 3?
set fmri(conmask2_3) 0

# Mask real contrast/F-test 2 with real contrast/F-test 4?
set fmri(conmask2_4) 0

# Mask real contrast/F-test 2 with real contrast/F-test 5?
set fmri(conmask2_5) 0

# Mask real contrast/F-test 2 with real contrast/F-test 6?
set fmri(conmask2_6) 0

# Mask real contrast/F-test 2 with real contrast/F-test 7?
set fmri(conmask2_7) 0

# Mask real contrast/F-test 2 with real contrast/F-test 8?
set fmri(conmask2_8) 0

# Mask real contrast/F-test 3 with real contrast/F-test 1?
set fmri(conmask3_1) 0

# Mask real contrast/F-test 3 with real contrast/F-test 2?
set fmri(conmask3_2) 0

# Mask real contrast/F-test 3 with real contrast/F-test 4?
set fmri(conmask3_4) 0

# Mask real contrast/F-test 3 with real contrast/F-test 5?
set fmri(conmask3_5) 0

# Mask real contrast/F-test 3 with real contrast/F-test 6?
set fmri(conmask3_6) 0

# Mask real contrast/F-test 3 with real contrast/F-test 7?
set fmri(conmask3_7) 0

# Mask real contrast/F-test 3 with real contrast/F-test 8?
set fmri(conmask3_8) 0

# Mask real contrast/F-test 4 with real contrast/F-test 1?
set fmri(conmask4_1) 0

# Mask real contrast/F-test 4 with real contrast/F-test 2?
set fmri(conmask4_2) 0

# Mask real contrast/F-test 4 with real contrast/F-test 3?
set fmri(conmask4_3) 0

# Mask real contrast/F-test 4 with real contrast/F-test 5?
set fmri(conmask4_5) 0

# Mask real contrast/F-test 4 with real contrast/F-test 6?
set fmri(conmask4_6) 0

# Mask real contrast/F-test 4 with real contrast/F-test 7?
set fmri(conmask4_7) 0

# Mask real contrast/F-test 4 with real contrast/F-test 8?
set fmri(conmask4_8) 0

# Mask real contrast/F-test 5 with real contrast/F-test 1?
set fmri(conmask5_1) 0

# Mask real contrast/F-test 5 with real contrast/F-test 2?
set fmri(conmask5_2) 0

# Mask real contrast/F-test 5 with real contrast/F-test 3?
set fmri(conmask5_3) 0

# Mask real contrast/F-test 5 with real contrast/F-test 4?
set fmri(conmask5_4) 0

# Mask real contrast/F-test 5 with real contrast/F-test 6?
set fmri(conmask5_6) 0

# Mask real contrast/F-test 5 with real contrast/F-test 7?
set fmri(conmask5_7) 0

# Mask real contrast/F-test 5 with real contrast/F-test 8?
set fmri(conmask5_8) 0

# Mask real contrast/F-test 6 with real contrast/F-test 1?
set fmri(conmask6_1) 0

# Mask real contrast/F-test 6 with real contrast/F-test 2?
set fmri(conmask6_2) 0

# Mask real contrast/F-test 6 with real contrast/F-test 3?
set fmri(conmask6_3) 0

# Mask real contrast/F-test 6 with real contrast/F-test 4?
set fmri(conmask6_4) 0

# Mask real contrast/F-test 6 with real contrast/F-test 5?
set fmri(conmask6_5) 0

# Mask real contrast/F-test 6 with real contrast/F-test 7?
set fmri(conmask6_7) 0

# Mask real contrast/F-test 6 with real contrast/F-test 8?
set fmri(conmask6_8) 0

# Mask real contrast/F-test 7 with real contrast/F-test 1?
set fmri(conmask7_1) 0

# Mask real contrast/F-test 7 with real contrast/F-test 2?
set fmri(conmask7_2) 0

# Mask real contrast/F-test 7 with real contrast/F-test 3?
set fmri(conmask7_3) 0

# Mask real contrast/F-test 7 with real contrast/F-test 4?
set fmri(conmask7_4) 0

# Mask real contrast/F-test 7 with real contrast/F-test 5?
set fmri(conmask7_5) 0

# Mask real contrast/F-test 7 with real contrast/F-test 6?
set fmri(conmask7_6) 0

# Mask real contrast/F-test 7 with real contrast/F-test 8?
set fmri(conmask7_8) 0

# Mask real contrast/F-test 8 with real contrast/F-test 1?
set fmri(conmask8_1) 0

# Mask real contrast/F-test 8 with real contrast/F-test 2?
set fmri(conmask8_2) 0

# Mask real contrast/F-test 8 with real contrast/F-test 3?
set fmri(conmask8_3) 0

# Mask real contrast/F-test 8 with real contrast/F-test 4?
set fmri(conmask8_4) 0

# Mask real contrast/F-test 8 with real contrast/F-test 5?
set fmri(conmask8_5) 0

# Mask real contrast/F-test 8 with real contrast/F-test 6?
set fmri(conmask8_6) 0

# Mask real contrast/F-test 8 with real contrast/F-test 7?
set fmri(conmask8_7) 0

### The rest is just fixed

# Do contrast masking at all?
set fmri(conmask1_1) 0

# Now options that don't appear in the GUI

# Alternative example_func image (not derived from input 4D dataset)
set fmri(alternative_example_func) ""

# Alternative (to BETting) mask image
set fmri(alternative_mask) ""

# Initial structural space registration initialisation transform
set fmri(init_initial_highres) ""

# Structural space registration initialisation transform
set fmri(init_highres) ""

# Standard space registration initialisation transform
set fmri(init_standard) ""


# For full FEAT analysis: overwrite existing .feat output dir?
set fmri(overwrite_yn) 1
