# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import SurfaceSmooth


def test_SurfaceSmooth_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        cortex=dict(
            argstr="--cortex",
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fwhm=dict(
            argstr="--fwhm %.4f",
            xor=["smooth_iters"],
        ),
        hemi=dict(
            argstr="--hemi %s",
            mandatory=True,
        ),
        in_file=dict(
            argstr="--sval %s",
            extensions=None,
            mandatory=True,
        ),
        out_file=dict(
            argstr="--tval %s",
            extensions=None,
            genfile=True,
        ),
        reshape=dict(
            argstr="--reshape",
        ),
        smooth_iters=dict(
            argstr="--smooth %d",
            xor=["fwhm"],
        ),
        subject_id=dict(
            argstr="--s %s",
            mandatory=True,
        ),
        subjects_dir=dict(),
    )
    inputs = SurfaceSmooth.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_SurfaceSmooth_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = SurfaceSmooth.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
