# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Camino-Trackvis allows interoperability between Camino and TrackVis."""
from .convert import Camino2Trackvis, Trackvis2Camino
