# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..stats import ActivationCount


def test_ActivationCount_inputs():
    input_map = dict(
        in_files=dict(
            mandatory=True,
        ),
        threshold=dict(
            mandatory=True,
        ),
    )
    inputs = ActivationCount.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ActivationCount_outputs():
    output_map = dict(
        acm_neg=dict(
            extensions=None,
        ),
        acm_pos=dict(
            extensions=None,
        ),
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = ActivationCount.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
